import com.bourre.data.libs.Config;
import com.bourre.data.libs.ConfigLoader;
import com.bourre.data.libs.GraphicLib;
import com.bourre.data.libs.GraphicLibEvent;
import com.bourre.data.libs.LibStack;
import com.bourre.events.BasicEvent;
import com.bourre.events.EventBroadcaster;
import com.bourre.log.Logger;
import com.bourre.log.PixlibDebug;
import com.bourre.log.PixlibStringifier;
import com.bourre.utils.LuminicTracer;
import com.bourre.visual.MovieClipHelper;

import test.controllers.Controller;
import test.events.EventList;
import test.tools.Debug;
import test.tools.ExpressInstall;
import test.uis.UIList;
	
class test.index.Index 
	extends MovieClipHelper
{
	// -- forcing import class, use to config.xml
	private var _classImport:Array = [com.bourre.data.collections.IndexedArray];
	
	private var _mcLoader:MovieClip;
	private var _tfLoader:TextField;
	
	private function Index( mc:MovieClip )
	{
		super(UIList.INDEX, mc);
		Debug.INFO( this + " launched!" );
		
		_buildLoader();
		_loadConfig();
	}
	
	private function _loadConfig( Void ) : Void
	{
		var libs:LibStack = new LibStack();
		libs.setAntiCache(true);
		// -- xml
		libs.enqueue( new ConfigLoader(), "config", "xml/config.xml");
		
		libs.addEventListener( LibStack.onLoadProgressEVENT, this );
		libs.addEventListener( LibStack.onLoadCompleteEVENT, this, _onConfigLoaded );
		libs.execute();
	}
	
	private function _loadAssets( Void ) : Void
	{
		var sPathPict:String = Config.getInstance()["path_pict"];
		// --
		var libs:LibStack = new LibStack();
		libs.setAntiCache(true);
		// -- UI
		libs.enqueue( _getGL(10), UIList.PICT, sPathPict+"pixLib.jpg");
		libs.enqueue( _getGL(11), UIList.BUTTON, "swf/button.swf");
		
		libs.addEventListener( LibStack.onLoadProgressEVENT, this );
		libs.addEventListener( LibStack.onLoadCompleteEVENT, this, _onAssetsLoaded );
		libs.execute();
	}
	
	private function _onConfigLoaded ( Void ) : Void
	{
		// -- protect Config object
		ConfigLoader.protectConfigObject( Config.getInstance() );
		_loadAssets();
	}
	
	private function _onAssetsLoaded( Void ) : Void
	{
		killLoader();
		// -- init FrontController
		// http://www.get-url.net/blog/?63
		Controller.getInstance().init();
		// -- broadcast the global event runSite, look Controller.init();
		EventBroadcaster.getInstance().broadcastEvent(new BasicEvent(EventList.runSiteEVENT));
	}
	
	private function _getGL( nDepth : Number, target : MovieClip ) : GraphicLib
	{
		return new GraphicLib( (target?target:view), nDepth, false );
	}
	
	private function _buildLoader( Void ) : Void
	{
		_mcLoader = view.createEmptyMovieClip("mcLoader", 0);
		_mcLoader.createTextField("txt", 1, 0, 0, 250, 50);
		_tfLoader = _mcLoader["txt"];
		_tfLoader.autoSize = _tfLoader.multiline = true;
		_tfLoader.selectable = false;
		_tfLoader.text = "initialization";
	}
	
	public function onLoadProgress( e : GraphicLibEvent ) : Void 
	{
		onProgress( e.getName(), e.getPerCent() );
	}
	
	public function killLoader() : Void
	{
		Debug.DEBUG("kill loader !!!");
		_mcLoader.removeMovieClip();
	}
	
	public function onProgress(sName:String, nPercent:Number) : Void
	{
		_tfLoader.text = "loading‚ \n";
		_tfLoader.text += sName+" "+nPercent.toString()+"%";
	}
	
	/**
	 * Main methode entry for MTASC
	 */
	public static function main( mc:MovieClip ) : Void 
	{
		Stage.align = "TL";
		Stage.scaleMode = "noScale";
		mc._focusRect = false;
		
		// -- Test FlashPlayer, swfObject
		if (ExpressInstall.init()) return;
		
		// -- add Logchannel Listener
		// -- http://www.get-url.net/blog/?15
		Logger.getInstance().addLogListener( LuminicTracer.getInstance(), PixlibDebug.channel );
		Logger.getInstance().addLogListener( LuminicTracer.getInstance(), Debug.channel );

		var o:Index = new Index( mc );
	}
	
	public function toString() : String 
	{
		return PixlibStringifier.stringify( this );
	}
}